/**
 * Shybox.js was creating some alignment related issues for User and Group pages, so created this file 
 * that works similar to ShyBox.js but changed logic of showing and creating a listbox. This implementation 
 * is more generic in terms of positioning HTML elements. This will find the appropriate position 
 * (irrespective of the browser) using offset-top and offset-left to position tghe element.
 * Note that most of the code from here is same as Shybox so at some point of time we should investigate 
 * which option to keep and which to get away with. This can also be made to work for ACL without much hassles.
 */
 
if (!window.EventUtils) alert("EditableComboBox requires EventUtils.js");
if (!window.StyleUtils) alert("EditableComboBox requires StyleUtils.js");


function EditableComboBox(){}

EditableComboBox.show = function(editableComboBox)
{
	function shim(shybox)
	{
		var shim = document.createElement('iframe');
		
		shim.src = "<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=JS_PATH --><!-- /WSLET -->dojo/dojo/resources/blank.html";
		shim.style.border = 'none';
		shim.style.position = 'absolute';

		shim.style.top = shybox.offsetTop + 'px';
		shim.style.left = shybox.offsetLeft + 'px';
		shim.style.width = shybox.offsetWidth + 'px';
		shim.style.height = shybox.offsetHeight + 'px';
		
		shybox.parentNode.insertBefore(shim, shybox);
		shybox.shim = shim;
	}

	var textBox = document.getElementById("textBox");
	var arrowImg = document.getElementById("arrowImg");
		
	editableComboBox.style.width = textBox.offsetWidth + arrowImg.offsetWidth + 'px'; 
	editableComboBox.style.top = arrowImg.offsetTop + arrowImg.offsetHeight + 'px';
	editableComboBox.style.left = arrowImg.offsetLeft - textBox.offsetWidth  + 'px';

	editableComboBox.style.display = 'block';

	if (document.all && !window.XMLHttpRequest)
	{
		// fix the show-through SELECTs in IE 6
		shim(editableComboBox);
	}

	if (editableComboBox.selectedIndex > -1)
	{
		editableComboBox.options[0].selected = true;
		editableComboBox.selectedIndex = -1;
	}

	var focusFunction = function()
	{
		editableComboBox.focus();
	};

	// this "delay" keeps IE from sending multiple blur events which hide the menu too often
	window.setTimeout(focusFunction, 0);
};

EditableComboBox.hide = function(editableComboBox)
{
	editableComboBox.selectedIndex = -1;	
	editableComboBox.style.display = 'none';
	
	if (editableComboBox.shim)
	{
		editableComboBox.parentNode.removeChild(editableComboBox.shim);
		editableComboBox.shim = null;
	}
};

EditableComboBox.editableComboBox_onBlur = function(e)
{
	var editableComboBox = EventUtils.getTarget(e);
	EditableComboBox.hide(editableComboBox);
};

EditableComboBox.getSelectedValue = function(editableComboBox)
{
	var index = editableComboBox.selectedIndex;
	
	if (index == -1) return null;

	return editableComboBox.options[editableComboBox.selectedIndex].value;
};

EditableComboBox.fireValueChange = function(editableComboBox)
{
	var listener = editableComboBox.valueListener;

	if (!listener) return;
	
	var value = EditableComboBox.getSelectedValue(editableComboBox)
	editableComboBox.selectedIndex = -1;
	
	if (value)
	{
		listener.value = value;
	}

	listener.focus();
};

EditableComboBox.setValue = function(editableComboBox, value)
{
	editableComboBox.valueListener.value = value;
};

EditableComboBox.text_onKeyDown = function(e)
{
	var target = EventUtils.getTarget(e);
	var event = EventUtils.getEvent(e);
	
	// show on 'down arrow'
	if (event.keyCode == 40) 
	{
		EditableComboBox.show(target.editableComboBox);
		EventUtils.preventDefault(e);
	}
};

EditableComboBox.editableComboBox_onKeydown = function(e)
{
	var editableComboBox = EventUtils.getTarget(e);
	var event = EventUtils.getEvent(e);
	
	switch (event.keyCode)
	{
		case 13:	// Enter
			EditableComboBox.fireValueChange(editableComboBox);
			EditableComboBox.hide(editableComboBox);
			break;
		case 27:	// esc
			EditableComboBox.hide(editableComboBox);
			break;
	}	
};

EditableComboBox.editableComboBox_onClick = function(e)
{
	var target = EventUtils.getTarget(e);

	// normalize click event between IE, Firefox
	var editableComboBox = (target.tagName == 'OPTION')
		? target.parentNode
		: target;

	EditableComboBox.fireValueChange(editableComboBox);
	EditableComboBox.hide(editableComboBox);
};

EditableComboBox.option_onMouseover = function(e)
{
	var option = EventUtils.getTarget(e);
	
	// this make it look more like an interactive menu, but doesn't work in IE
	option.selected = true;
};

EditableComboBox._notifyIfTextChanged = function(e)
{
	var textbox = EventUtils.getTarget(e);
	
	if (textbox._EditableComboBox_onChangeListener)
	{
		if (textbox._EditableComboBox_initialValue
			!= textbox.value)
		{
			textbox._EditableComboBox_onChangeListener();
		}
	}
};

/**
 * Convert a regular listbox (SELECT) into a editableComboBox, which is similar to a combobox.
 * @listbox the listbox to convert
 * @optionalInitialValue the initial value to show in the textfield.
 * @optionalOnInitialChangeListener a listener function called after a blur when the text value
 *		has changed from the initial value.
 */
EditableComboBox.convertToEditableComboBox = function(listbox, optionalInitialValue, optionalOnInitialChangeListener)
{	
	var frag = document.createDocumentFragment();

	// 2) create the input for the user to edit
	var textbox = document.createElement('INPUT');
	textbox.type = 'text';
	textbox.id = 'textBox';
	StyleUtils.addClass(textbox, 'editableComboBox-text');	
	
	if (optionalInitialValue)
	{
		textbox._EditableComboBox_initialValue = optionalInitialValue;
		textbox.value = optionalInitialValue;
	}
	
	if (optionalOnInitialChangeListener)
	{
		EventUtils.addEvent(textbox, 'blur', EditableComboBox._notifyIfTextChanged);
		textbox._EditableComboBox_onChangeListener = optionalOnInitialChangeListener;
	}

	EventUtils.addEvent(textbox, 'keydown', EditableComboBox.text_onKeyDown);
	// give the textbox and easy reference to the select box to ease showing it later
	textbox.editableComboBox = listbox;
	listbox.valueListener = textbox;
	if (listbox.size < 2) listbox.size = 10;
	
	// 3) massage the select box
	StyleUtils.addClass(listbox, 'editableComboBox');
	listbox.style.display = 'none';

	EventUtils.addEvent(listbox, 'click', EditableComboBox.editableComboBox_onClick);
	EventUtils.addEvent(listbox, 'blur', EditableComboBox.editableComboBox_onBlur);
	EventUtils.addEvent(listbox, 'keydown', EditableComboBox.editableComboBox_onKeydown);

	var options = listbox.options;
	
	for (var i = 0; i < options.length; i++)
	{
		// this only works in Firefox, but it makes the options work like menu options
		// with auto highlighting
		EventUtils.addEvent(options[i], 'mouseover', EditableComboBox.option_onMouseover, false);
	}
	
	// 4) add arrow control to reveal the hidden select	
	var showFunction = function(e)
	{
		EditableComboBox.show(listbox);
	};

	var arrowImg = document.createElement('IMG');
	arrowImg.id = 'arrowImg';
	arrowImg.src = '<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=PTIMAGES_PATH --><!-- /WSLET -->shybox_arrow.gif';
	
	StyleUtils.addClass(arrowImg, 'editableComboBox-arrow');
		
	EventUtils.addEvent(arrowImg, 'click', showFunction);
	
	// 5) assemble and return
	frag.appendChild(textbox);
	frag.appendChild(listbox);
	frag.appendChild(arrowImg);

	listbox._EditableComboBox_arrow = arrowImg;
	listbox._EditableComboBox_textbox = textbox;
	
	return frag;
};
